package samples.webcenter.portlets.iframe.blog;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


public class BlogPortlet extends GenericPortlet {

    public static String URL = "url";
    public static String TYPE = "type"; // Personal = 0, Community = 1
    public static String SCOPE =
        "scope"; // May be used as name for personal blogs, scope for community blogs
    public static String THEME = "theme";
    public static String INLINE = "inline";

    public static String[] prefNames = { URL, TYPE, SCOPE, THEME, INLINE };

    public void _doIt(RenderRequest request, RenderResponse response,
                      String path) throws PortletException, IOException {

        response.setContentType("text/html");

        // Make preferences available to JSPs
        PortletPreferences prefs = request.getPreferences();
        PortletSession session = request.getPortletSession();
        String name = null;
        String value = null;

        for (int i = 0; i < prefNames.length; i++) {
            name = prefNames[i];

            value = prefs.getValue(name, null);

            if (request.getUserPrincipal() == null) {
                value = (String)session.getAttribute(name);
            } else {
                value = prefs.getValue(name, null);
                session.setAttribute(name, value);
            }

            request.setAttribute("blog_" + name, value);
        }

        // Construct the iFrame src
        String url = (String)session.getAttribute(URL);
        url = url != null ? url : "";
        String type = (String)session.getAttribute(TYPE);
        type = type != null ? type : "";
        String scope = (String)session.getAttribute(SCOPE);
        scope = scope != null ? scope : "";
        String theme = (String)session.getAttribute(THEME);
        theme = theme != null ? theme : "";
        String inline = (String)session.getAttribute(INLINE);
        inline = inline != null ? inline : "";

        if (url.length() > 0) {
            url =
(url.lastIndexOf("/") == url.length() - 1) ? url = url.substring(0,
                                                                 url.length() -
                                                                 1) : url;
            System.out.println("iFrame Blog org URL = **" + url + "**");
            String cleanUrl = url.trim();
            System.out.println("iFrame Blog cleaned URL = **" + cleanUrl + "**");
            //            String blogSrc = url + "/blog/list.jz";
            String blogSrc = cleanUrl + "/blog/list.jz";
            System.out.println("blogSrc iFrame = " + blogSrc);
            blogSrc +=
                    "?" + (("0".equals(type)) ? "name=" + scope : "domain=" +
                           scope);
            blogSrc += "&theme=" + theme;
            blogSrc += "&inline=" + inline;
            request.setAttribute("blog_src", blogSrc);
        }

        request.setAttribute("blog_action",
                             session.getAttribute("blog_action"));
        session.removeAttribute("blog_action");

        PortletContext context = getPortletConfig().getPortletContext();
        context.getRequestDispatcher(path).include(request, response);
    }

    public void doView(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/blog/view.jsp");
    }

    public void doEdit(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       java.io.IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/blog/edit.jsp");
    }


    public void doHelp(RenderRequest request,
                       RenderResponse response) throws PortletException,
                                                       java.io.IOException {

        this._doIt(request, response,
                   "/samples/webcenter/portlets/iframe/blog/help.jsp");
    }

    public void processAction(ActionRequest request,
                              ActionResponse response) throws PortletException,
                                                              java.io.IOException {

        if (request.getPortletMode().equals(PortletMode.VIEW)) {
            this.parsePrefs(request, response);
            request.getPortletSession().setAttribute("blog_action",
                                                     request.getParameter("blog_action"));
        } else {
            if (request.getPortletMode().equals(PortletMode.EDIT)) {
                this.parsePrefs(request, response);
                response.setPortletMode(PortletMode.VIEW);
            }
        }
    }

    public void parsePrefs(PortletRequest request,
                           PortletResponse response) throws PortletException,
                                                            java.io.IOException {

        PortletPreferences prefs = request.getPreferences();
        String name = null;
        String value = null;

        PortletSession session = request.getPortletSession();

        for (int i = 0; i < prefNames.length; i++) {
            name = prefNames[i];
            value = request.getParameter("blog_" + name);
            if (value != null) {
                prefs.setValue(name, value);
                session.setAttribute(name, value);
            }
        }

        try {
            prefs.store();
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }
}
